package de.intevation.flys.exports;

import org.apache.log4j.Logger;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.Range;
import org.jfree.data.xy.XYSeries;

import org.w3c.dom.Document;

import de.intevation.artifacts.Artifact;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WQKms;


/**
 * An OutGenerator that generates cross section graphs.
 */
public class CrossSectionGenerator
extends      XYChartGenerator
implements   FacetTypes
{
    /** The logger that is used in this generator. */
    private static Logger logger =
        Logger.getLogger(CrossSectionGenerator.class);

    public static final String I18N_CHART_TITLE =
        "chart.cross_section.title";

    public static final String I18N_CHART_SUBTITLE =
        "chart.cross_section.subtitle";

    public static final String I18N_XAXIS_LABEL =
        "chart.cross_section.xaxis.label";

    public static final String I18N_YAXIS_LABEL =
        "chart.cross_section.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT = "Querprofildiagramm";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Abstand [m]";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "W [NN + m]";


    /** Trivial Constructor. */
    public CrossSectionGenerator() {
        super();
    }


    /**
     * Get localized chart title.
     */
    protected String getChartTitle() {
        // TODO get river etc for localized heading
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }


    /**
     * Add localized subtitle to chart.
     */
    @Override
    protected void addSubtitles(JFreeChart chart) {
        double[] dist = getRange();

        Object[] args = new Object[] {
            getRiverName(),
            dist[0],
            dist[1]
        };

        String subtitle = msg(I18N_CHART_SUBTITLE, "", args);
        chart.addSubtitle(new TextTitle(subtitle));
    }


    /**
     * Get localized X Axis label.
     */
    protected String getXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }


    /**
     * Get localized Y Axis label.
     */
    protected String getYAxisLabel() {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }


    protected void adjustAxes(XYPlot plot) {
        super.adjustAxes(plot);

        NumberAxis qAxis = new NumberAxis(
            msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT));

        plot.setRangeAxis(1, qAxis);
    }


    /**
     * Overrides XYChartGenerators.zoomY() to include the 0 value on the Q axis.
     */
    @Override
    protected boolean zoomY(XYPlot plot, ValueAxis axis, Range range, Range x) {
        if (plot.getRangeAxisIndex(axis) == 1) {
            // we want the Q axis to start at 0 if no zooming has been done
            range = new Range(0d, range.getUpperBound());
        }

        return super.zoomY(plot, axis, range, x);
    }


    /**
     * Let one facet do its job.
     */
    public void doOut(Artifact artifact, Facet facet, Document attr) {
        String name = facet.getName();

        logger.debug("CrossSectionGenerator.doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        FLYSArtifact flys = (FLYSArtifact) artifact;
        Facet        f    = flys.getNativeFacet(facet);

        if (f == null) {
            return;
        }

        if (name.equals(CROSS_SECTION)) {
            doCrossSectionOut(f.getData(artifact, context), attr);
        }
        else if (name.equals(CROSS_SECTION_WATER_LINE)) {
            doCrossSectionWaterLineOut(f.getData(artifact, context), attr);
        }
        else {
            logger.warn("CrossSection.doOut: Unknown facet name: " + name);
            return;
        }
    }


    /**
     * Do cross sections waterline out.
     *
     * @param theme Theme for the data series.
     */
    protected void doCrossSectionWaterLineOut(Object o, Document theme) {
        logger.debug("CrossSectionGenerator.doCrossSectionWaterLineOut");
        // TODO Series should get name like "Q=22.0"
        XYSeries series = new StyledXYSeries("Q= ... ", theme);

        double[][] a = (double [][]) o;
        double [] pxs = a[0];
        for (int i = 0; i < pxs.length; i++) {
            series.add (a[0][i], a[1][i]);
        }
        addFirstAxisSeries(series);
    }


    /**
     * Do cross sections out.
     *
     * @param theme Theme for the data series.
     */
    protected void doCrossSectionOut(Object o, Document theme) {
        logger.debug("CrossSectionGenerator.doCrossSectionOut");

        // TODO Series should get name of selected cross-section (e.g. 
        // "0-93.1(1990-2002).PRF") (exposed as getCrossSectionName in
        //  WINFOArtifact)
        XYSeries series = new StyledXYSeries("aliquide", theme);

        double[][] a = (double [][]) o;
        double [] pxs = a[0];
        for (int i = 0; i < pxs.length; i++) {
            series.add (a[0][i], a[1][i]);
        }
        addFirstAxisSeries(series);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
